@@ -110,6 +110,113 @@ if { $bCheckIPsPassed != 1 } {
 # DESIGN PROCs
 ##################################################################
 
+# Hierarchical cell: hwr_inStream
+proc create_hier_cell_hwr_inStream { parentCell nameHier } {
+
+  variable script_folder
+
+  if { $parentCell eq "" || $nameHier eq "" } {
+     catch {common::send_msg_id "BD_TCL-102" "ERROR" "create_hier_cell_hwr_inStream() - Empty argument(s)!"}
+     return
+  }
+
+  # Get object for parentCell
+  set parentObj [get_bd_cells $parentCell]
+  if { $parentObj == "" } {
+     catch {common::send_msg_id "BD_TCL-100" "ERROR" "Unable to find parent cell <$parentCell>!"}
+     return
+  }
+
+  # Make sure parentObj is hier blk
+  set parentType [get_property TYPE $parentObj]
+  if { $parentType ne "hier" } {
+     catch {common::send_msg_id "BD_TCL-101" "ERROR" "Parent <$parentObj> has TYPE = <$parentType>. Expected to be <hier>."}
+     return
+  }
+
+  # Save current instance; Restore later
+  set oldCurInst [current_bd_instance .]
+
+  # Set parent object as current
+  current_bd_instance $parentObj
+
+  # Create cell and set as current instance
+  set hier_obj [create_bd_cell -type hier $nameHier]
+  current_bd_instance $hier_obj
+
+  # Create interface pins
+  for {set i 0} {$i < ${::AIT::num_accs}} {incr i} {
+      create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:axis_rtl:1.0 S${i}_AXIS
+  }
+  create_bd_intf_pin -mode Master -vlnv xilinx.com:interface:axis_rtl:1.0 cmdout_in
+  create_bd_intf_pin -mode Master -vlnv xilinx.com:interface:axis_rtl:1.0 spawn_in
+  create_bd_intf_pin -mode Master -vlnv xilinx.com:interface:axis_rtl:1.0 taskwait_in
+  if {${::AIT::lock_hwruntime}} {
+      create_bd_intf_pin -mode Master -vlnv xilinx.com:interface:axis_rtl:1.0 lock_in
+  }
+
+  # Create pins
+  create_bd_pin -dir I -type clk clk
+  create_bd_pin -dir I -type rst interconnect_aresetn
+  create_bd_pin -dir I -type rst peripheral_aresetn
+
+  # Restore current instance
+  current_bd_instance $oldCurInst
+}
+
+# Hierarchical cell: hwr_outStream
+proc create_hier_cell_hwr_outStream { parentCell nameHier } {
+
+  variable script_folder
+
+  if { $parentCell eq "" || $nameHier eq "" } {
+     catch {common::send_msg_id "BD_TCL-102" "ERROR" "create_hier_cell_hwr_outStream() - Empty argument(s)!"}
+     return
+  }
+
+  # Get object for parentCell
+  set parentObj [get_bd_cells $parentCell]
+  if { $parentObj == "" } {
+     catch {common::send_msg_id "BD_TCL-100" "ERROR" "Unable to find parent cell <$parentCell>!"}
+     return
+  }
+
+  # Make sure parentObj is hier blk
+  set parentType [get_property TYPE $parentObj]
+  if { $parentType ne "hier" } {
+     catch {common::send_msg_id "BD_TCL-101" "ERROR" "Parent <$parentObj> has TYPE = <$parentType>. Expected to be <hier>."}
+     return
+  }
+
+  # Save current instance; Restore later
+  set oldCurInst [current_bd_instance .]
+
+  # Set parent object as current
+  current_bd_instance $parentObj
+
+  # Create cell and set as current instance
+  set hier_obj [create_bd_cell -type hier $nameHier]
+  current_bd_instance $hier_obj
+
+  # Create interface pins
+  for {set i 0} {$i < ${::AIT::num_accs}} {incr i} {
+      create_bd_intf_pin -mode Master -vlnv xilinx.com:interface:axis_rtl:1.0 M${i}_AXIS
+  }
+  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:axis_rtl:1.0 cmdin_out
+  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:axis_rtl:1.0 spawn_out
+  create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:axis_rtl:1.0 taskwait_out
+  if {${::AIT::lock_hwruntime}} {
+      create_bd_intf_pin -mode Slave -vlnv xilinx.com:interface:axis_rtl:1.0 lock_out
+  }
+
+  # Create pins
+  create_bd_pin -dir I -type clk clk
+  create_bd_pin -dir I -type rst interconnect_aresetn
+  create_bd_pin -dir I -type rst peripheral_aresetn
+
+  # Restore current instance
+  current_bd_instance $oldCurInst
+}
 
 # Hierarchical cell: Hardware_Runtime
 proc create_hier_cell_Hardware_Runtime { parentCell nameHier } {
@@ -155,32 +262,94 @@ proc create_hier_cell_Hardware_Runtime { parentCell nameHier } {
   create_bd_pin -dir O managed_aresetn
   create_bd_pin -dir I -type rst peripheral_aresetn
 
+  # Create instance: hwr_inStream
+  create_hier_cell_hwr_inStream [current_bd_instance .] hwr_inStream
+
+  # Create instance: hwr_outStream
+  create_hier_cell_hwr_outStream [current_bd_instance .] hwr_outStream
+
   # Create instance: GP_Inter, and set properties
   set GP_Inter [ create_bd_cell -type ip -vlnv xilinx.com:ip:axi_interconnect GP_Inter ]
-    if {$::enable_spawn_queues} {
-        set_property -dict [ list \
-            CONFIG.M00_HAS_REGSLICE {4} \
-            CONFIG.M01_HAS_REGSLICE {4} \
-            CONFIG.M02_HAS_REGSLICE {4} \
-            CONFIG.M03_HAS_REGSLICE {4} \
-            CONFIG.M04_HAS_REGSLICE {4} \
-            CONFIG.NUM_MI {5} \
-            CONFIG.S00_HAS_REGSLICE {4} \
-            CONFIG.STRATEGY {1} \
-        ] $GP_Inter
-    } else {
-        set_property -dict [ list \
-            CONFIG.M00_HAS_REGSLICE {4} \
-            CONFIG.M01_HAS_REGSLICE {4} \
-            CONFIG.M02_HAS_REGSLICE {4} \
-            CONFIG.NUM_MI {3} \
-            CONFIG.S00_HAS_REGSLICE {4} \
-            CONFIG.STRATEGY {1} \
-        ] $GP_Inter
-    }
+  if {$::AIT::enable_spawn_queues} {
+    set_property -dict [ list \
+     CONFIG.M00_HAS_REGSLICE {4} \
+     CONFIG.M01_HAS_REGSLICE {4} \
+     CONFIG.M02_HAS_REGSLICE {4} \
+     CONFIG.M03_HAS_REGSLICE {4} \
+     CONFIG.M04_HAS_REGSLICE {4} \
+     CONFIG.NUM_MI {5} \
+     CONFIG.S00_HAS_REGSLICE {4} \
+     CONFIG.STRATEGY {1} \
+    ] $GP_Inter
+  } else {
+    set_property -dict [ list \
+     CONFIG.M00_HAS_REGSLICE {4} \
+     CONFIG.M01_HAS_REGSLICE {4} \
+     CONFIG.M02_HAS_REGSLICE {4} \
+     CONFIG.NUM_MI {3} \
+     CONFIG.S00_HAS_REGSLICE {4} \
+     CONFIG.STRATEGY {1} \
+    ] $GP_Inter
+  }
+
+  # Create instance: axis_cmdin_TID, and set properties
+  set axis_cmdin_TID [create_bd_cell -type ip -vlnv xilinx.com:ip:axis_subset_converter axis_cmdin_TID]
+  set_property -dict [ list \
+   CONFIG.M_TID_WIDTH.VALUE_SRC USER \
+   CONFIG.M_TID_WIDTH {1} \
+   #CONFIG.TID_REMAP "1'b1"
+  ] $axis_cmdin_TID
+
+  # Create instance: axis_spawn_TID, and set properties
+  set axis_spawn_TID [create_bd_cell -type ip -vlnv xilinx.com:ip:axis_subset_converter axis_spawn_TID]
+  set_property -dict [ list \
+   CONFIG.M_TID_WIDTH.VALUE_SRC USER \
+   CONFIG.M_TID_WIDTH {1} \
+   CONFIG.TID_REMAP "1'b1" \
+  ] $axis_spawn_TID
+
+  # Create instance: axis_taskwait_TID, and set properties
+  set axis_taskwait_TID [create_bd_cell -type ip -vlnv xilinx.com:ip:axis_subset_converter axis_taskwait_TID]
+  set_property -dict [ list \
+   CONFIG.M_TID_WIDTH.VALUE_SRC USER \
+   CONFIG.M_TID_WIDTH {1} \
+   CONFIG.TID_REMAP "1'b1" \
+  ] $axis_taskwait_TID
+
+  if {${::AIT::lock_hwruntime}} {
+    # Create instance: axis_lock_TID, and set properties
+    set axis_lock_TID [create_bd_cell -type ip -vlnv xilinx.com:ip:axis_subset_converter axis_lock_TID]
+    set_property -dict [ list \
+     CONFIG.M_TID_WIDTH.VALUE_SRC USER \
+     CONFIG.M_TID_WIDTH {1} \
+     #CONFIG.TID_REMAP "1'b1"
+    ] $axis_lock_TID
+  }
 
   # Create instance: Picos_OmpSs_Manager, and set properties
   set Picos_OmpSs_Manager [ create_bd_cell -type ip -vlnv bsc:ompss:picosompssmanager Picos_OmpSs_Manager ]
+  set_property -dict [ list \
+   CONFIG.PICOS_ARGS ${::AIT::picos_args_hash} \
+   CONFIG.MAX_ACCS [expr max(${::AIT::num_accs}, 2)] \
+   CONFIG.MAX_ACC_TYPES [expr max([llength ${::AIT::accs}], 2)] \
+   CONFIG.MAX_ACC_CREATORS [expr max(${::AIT::num_acc_creators}, 2)] \
+   CONFIG.CMDIN_SUBQUEUE_LEN ${::AIT::cmdInSubqueue_len} \
+   CONFIG.CMDOUT_SUBQUEUE_LEN ${::AIT::cmdOutSubqueue_len} \
+   CONFIG.ENABLE_SPAWN_QUEUES ${::AIT::enable_spawn_queues} \
+  ] $Picos_OmpSs_Manager
+
+  if {${::AIT::enable_spawn_queues}} {
+    set_property -dict [list \
+      CONFIG.SPAWNIN_QUEUE_LEN ${::AIT::spawnInQueue_len} \
+      CONFIG.SPAWNOUT_QUEUE_LEN ${::AIT::spawnOutQueue_len} \
+    ] $Picos_OmpSs_Manager
+  }
+
+  if {${::AIT::lock_hwruntime}} {
+    set_property -dict [list \
+      CONFIG.LOCK_SUPPORT {1} \
+    ] $Picos_OmpSs_Manager
+  }
 
   # Create instance: cmdInQueue, and set properties
   set cmdInQueue [ create_bd_cell -type ip -vlnv xilinx.com:ip:blk_mem_gen cmdInQueue ]
@@ -203,7 +372,7 @@ proc create_hier_cell_Hardware_Runtime { parentCell nameHier } {
    CONFIG.Use_Byte_Write_Enable {true} \
    CONFIG.Use_RSTA_Pin {false} \
    CONFIG.Use_RSTB_Pin {true} \
-   CONFIG.Write_Depth_A {1024} \
+   CONFIG.Write_Depth_A [expr ${::AIT::cmdInSubqueue_len}*max(${::AIT::num_accs}, 2)] \
    CONFIG.Write_Width_A {64} \
    CONFIG.Write_Width_B {32} \
    CONFIG.use_bram_block {Stand_Alone} \
@@ -236,7 +405,7 @@ proc create_hier_cell_Hardware_Runtime { parentCell nameHier } {
    CONFIG.Use_Byte_Write_Enable {true} \
    CONFIG.Use_RSTA_Pin {false} \
    CONFIG.Use_RSTB_Pin {true} \
-   CONFIG.Write_Depth_A {1024} \
+   CONFIG.Write_Depth_A [expr ${::AIT::cmdOutSubqueue_len}*max(${::AIT::num_accs}, 2)] \
    CONFIG.Write_Width_A {64} \
    CONFIG.Write_Width_B {32} \
    CONFIG.use_bram_block {Stand_Alone} \
@@ -256,7 +425,7 @@ proc create_hier_cell_Hardware_Runtime { parentCell nameHier } {
    CONFIG.C_GPIO_WIDTH {1} \
  ] $hwruntime_rst
 
-if {$::enable_spawn_queues} {
+if {$::AIT::enable_spawn_queues} {
     # Create instance: spawnInQueue, and set properties
     set spawnInQueue [ create_bd_cell -type ip -vlnv xilinx.com:ip:blk_mem_gen spawnInQueue ]
     set_property -dict [ list \
@@ -278,7 +447,7 @@ if {$::enable_spawn_queues} {
         CONFIG.Use_Byte_Write_Enable {true} \
         CONFIG.Use_RSTA_Pin {false} \
         CONFIG.Use_RSTB_Pin {true} \
-        CONFIG.Write_Depth_A {1024} \
+        CONFIG.Write_Depth_A ${::AIT::spawnInQueue_len} \
         CONFIG.Write_Width_A {64} \
         CONFIG.Write_Width_B {32} \
         CONFIG.use_bram_block {Stand_Alone} \
@@ -311,7 +480,7 @@ if {$::enable_spawn_queues} {
         CONFIG.Use_Byte_Write_Enable {true} \
         CONFIG.Use_RSTA_Pin {false} \
         CONFIG.Use_RSTB_Pin {true} \
-        CONFIG.Write_Depth_A {1024} \
+        CONFIG.Write_Depth_A ${::AIT::spawnOutQueue_len} \
         CONFIG.Write_Width_A {64} \
         CONFIG.Write_Width_B {32} \
         CONFIG.use_bram_block {Stand_Alone} \
@@ -325,6 +494,20 @@ if {$::enable_spawn_queues} {
 }
 
   # Create interface connections
+  connect_bd_intf_net [get_bd_intf_pins Picos_OmpSs_Manager/cmdout_in] [get_bd_intf_pins hwr_inStream/cmdout_in]
+  connect_bd_intf_net [get_bd_intf_pins Picos_OmpSs_Manager/cmdin_out] [get_bd_intf_pins axis_cmdin_TID/S_AXIS]
+  connect_bd_intf_net [get_bd_intf_pins hwr_outStream/cmdin_out] [get_bd_intf_pins axis_cmdin_TID/M_AXIS]
+  connect_bd_intf_net [get_bd_intf_pins Picos_OmpSs_Manager/spawn_in] [get_bd_intf_pins hwr_inStream/spawn_in]
+  connect_bd_intf_net [get_bd_intf_pins Picos_OmpSs_Manager/spawn_out] [get_bd_intf_pins axis_spawn_TID/S_AXIS]
+  connect_bd_intf_net [get_bd_intf_pins hwr_outStream/spawn_out] [get_bd_intf_pins axis_spawn_TID/M_AXIS]
+  connect_bd_intf_net [get_bd_intf_pins Picos_OmpSs_Manager/taskwait_in] [get_bd_intf_pins hwr_inStream/taskwait_in]
+  connect_bd_intf_net [get_bd_intf_pins Picos_OmpSs_Manager/taskwait_out] [get_bd_intf_pins axis_taskwait_TID/S_AXIS]
+  connect_bd_intf_net [get_bd_intf_pins hwr_outStream/taskwait_out] [get_bd_intf_pins axis_taskwait_TID/M_AXIS]
+  if {${::AIT::lock_hwruntime}} {
+    connect_bd_intf_net [get_bd_intf_pins Picos_OmpSs_Manager/lock_in] [get_bd_intf_pins hwr_inStream/lock_in]
+    connect_bd_intf_net [get_bd_intf_pins Picos_OmpSs_Manager/lock_out] [get_bd_intf_pins axis_lock_TID/S_AXIS]
+    connect_bd_intf_net [get_bd_intf_pins hwr_outStream/lock_out] [get_bd_intf_pins axis_lock_TID/M_AXIS]
+  }
   connect_bd_intf_net -intf_net GP_Inter_M00_AXI [get_bd_intf_pins GP_Inter/M00_AXI] [get_bd_intf_pins cmdInQueue_BRAM_Ctrl/S_AXI]
   connect_bd_intf_net -intf_net GP_Inter_M01_AXI [get_bd_intf_pins GP_Inter/M01_AXI] [get_bd_intf_pins cmdOutQueue_BRAM_Ctrl/S_AXI]
   connect_bd_intf_net -intf_net GP_Inter_M02_AXI [get_bd_intf_pins GP_Inter/M02_AXI] [get_bd_intf_pins hwruntime_rst/S_AXI]
@@ -334,7 +517,7 @@ if {$::enable_spawn_queues} {
   connect_bd_intf_net -intf_net S_AXI_GP_1 [get_bd_intf_pins S_AXI_GP] [get_bd_intf_pins GP_Inter/S00_AXI]
   connect_bd_intf_net -intf_net cmdInQueue_BRAM_Ctrl_BRAM_PORTA [get_bd_intf_pins cmdInQueue/BRAM_PORTB] [get_bd_intf_pins cmdInQueue_BRAM_Ctrl/BRAM_PORTA]
   connect_bd_intf_net -intf_net cmdOutQueue_BRAM_Ctrl_BRAM_PORTA [get_bd_intf_pins cmdOutQueue/BRAM_PORTB] [get_bd_intf_pins cmdOutQueue_BRAM_Ctrl/BRAM_PORTA]
-  if {$::enable_spawn_queues} {
+  if {$::AIT::enable_spawn_queues} {
     connect_bd_intf_net -intf_net Picos_OmpSs_Manager_spawnInQueue [get_bd_intf_pins Picos_OmpSs_Manager/spawnin_queue] [get_bd_intf_pins spawnInQueue/BRAM_PORTA]
     connect_bd_intf_net -intf_net Picos_OmpSs_Manager_spawnOutQueue [get_bd_intf_pins Picos_OmpSs_Manager/spawnout_queue] [get_bd_intf_pins spawnOutQueue/BRAM_PORTA]
     connect_bd_intf_net -intf_net spawnInQueue_BRAM_Ctrl_BRAM_PORTA [get_bd_intf_pins spawnInQueue/BRAM_PORTB] [get_bd_intf_pins spawnInQueue_BRAM_Ctrl/BRAM_PORTA]
@@ -346,11 +529,11 @@ if {$::enable_spawn_queues} {
   }
 
   # Create port connections
-  connect_bd_net -net Picos_OmpSs_Manager_managed_aresetn [get_bd_pins managed_aresetn] [get_bd_pins Picos_OmpSs_Manager/managed_aresetn]
-  connect_bd_net -net aclk_1 [get_bd_pins aclk] [get_bd_pins GP_Inter/ACLK] [get_bd_pins GP_Inter/M00_ACLK] [get_bd_pins GP_Inter/M01_ACLK] [get_bd_pins GP_Inter/M02_ACLK] [get_bd_pins GP_Inter/M03_ACLK] [get_bd_pins GP_Inter/M04_ACLK] [get_bd_pins GP_Inter/S00_ACLK] [get_bd_pins Picos_OmpSs_Manager/aclk] [get_bd_pins cmdInQueue_BRAM_Ctrl/s_axi_aclk] [get_bd_pins cmdOutQueue_BRAM_Ctrl/s_axi_aclk] [get_bd_pins hwruntime_rst/s_axi_aclk]
+  connect_bd_net -net Picos_OmpSs_Manager_managed_aresetn [get_bd_pins managed_aresetn] [get_bd_pins Picos_OmpSs_Manager/managed_aresetn] [get_bd_pins axis_spawn_TID/aresetn] [get_bd_pins axis_taskwait_TID/aresetn] [get_bd_pins axis_cmdin_TID/aresetn]
+  connect_bd_net -net aclk_1 [get_bd_pins aclk] [get_bd_pins GP_Inter/ACLK] [get_bd_pins GP_Inter/M00_ACLK] [get_bd_pins GP_Inter/M01_ACLK] [get_bd_pins GP_Inter/M02_ACLK] [get_bd_pins GP_Inter/M03_ACLK] [get_bd_pins GP_Inter/M04_ACLK] [get_bd_pins GP_Inter/S00_ACLK] [get_bd_pins Picos_OmpSs_Manager/aclk] [get_bd_pins cmdInQueue_BRAM_Ctrl/s_axi_aclk] [get_bd_pins cmdOutQueue_BRAM_Ctrl/s_axi_aclk] [get_bd_pins hwruntime_rst/s_axi_aclk] [get_bd_pins hwr_inStream/clk] [get_bd_pins hwr_outStream/clk] [get_bd_pins axis_spawn_TID/aclk] [get_bd_pins axis_taskwait_TID/aclk] [get_bd_pins axis_cmdin_TID/aclk]
   connect_bd_net -net hwruntime_rst_gpio_io_o [get_bd_pins Picos_OmpSs_Manager/ps_rst] [get_bd_pins hwruntime_rst/gpio_io_o]
-  connect_bd_net -net interconnect_aresetn_1 [get_bd_pins interconnect_aresetn] [get_bd_pins GP_Inter/ARESETN] [get_bd_pins Picos_OmpSs_Manager/interconnect_aresetn]
-  connect_bd_net -net peripheral_aresetn_1 [get_bd_pins peripheral_aresetn] [get_bd_pins GP_Inter/M00_ARESETN] [get_bd_pins GP_Inter/M01_ARESETN] [get_bd_pins GP_Inter/M02_ARESETN] [get_bd_pins GP_Inter/M03_ARESETN] [get_bd_pins GP_Inter/M04_ARESETN] [get_bd_pins GP_Inter/S00_ARESETN] [get_bd_pins Picos_OmpSs_Manager/peripheral_aresetn] [get_bd_pins cmdInQueue_BRAM_Ctrl/s_axi_aresetn] [get_bd_pins cmdOutQueue_BRAM_Ctrl/s_axi_aresetn] [get_bd_pins hwruntime_rst/s_axi_aresetn]
+  connect_bd_net -net interconnect_aresetn_1 [get_bd_pins interconnect_aresetn] [get_bd_pins GP_Inter/ARESETN] [get_bd_pins Picos_OmpSs_Manager/interconnect_aresetn] [get_bd_pins hwr_inStream/interconnect_aresetn] [get_bd_pins hwr_outStream/interconnect_aresetn]
+  connect_bd_net -net peripheral_aresetn_1 [get_bd_pins peripheral_aresetn] [get_bd_pins GP_Inter/M00_ARESETN] [get_bd_pins GP_Inter/M01_ARESETN] [get_bd_pins GP_Inter/M02_ARESETN] [get_bd_pins GP_Inter/M03_ARESETN] [get_bd_pins GP_Inter/M04_ARESETN] [get_bd_pins GP_Inter/S00_ARESETN] [get_bd_pins Picos_OmpSs_Manager/peripheral_aresetn] [get_bd_pins cmdInQueue_BRAM_Ctrl/s_axi_aresetn] [get_bd_pins cmdOutQueue_BRAM_Ctrl/s_axi_aresetn] [get_bd_pins hwruntime_rst/s_axi_aresetn] [get_bd_pins hwr_inStream/peripheral_aresetn] [get_bd_pins hwr_outStream/peripheral_aresetn]
 
   # Restore current instance
   current_bd_instance $oldCurInst
